'use strict';

const MagicString = require('magic-string');
const vite = require('vite');

class Context {
  constructor(options) {
    this.options = options;
    this.blockRegex = /^.*?#v-if(n?)def\s*(\S+).*[\r\n]{1,2}([\s\S]+?)\s*.*?#v-endif.*?$/gm;
    this.filter = vite.createFilter(this.options.include, this.options.exclude);
    this.logger = vite.createLogger("info", {
      prefix: "[vite-plugin-conditional-compile]"
    });
  }
  setEnv(config) {
    this.env = config.env;
  }
  transform(code, id) {
    try {
      if (!this.filter(id))
        return;
      const source = this.createSource(code, id);
      source.replace(this.blockRegex, this.blockReplaceHandler.bind(this));
      if (source.hasChanged()) {
        return {
          code: source.toString(),
          map: source.generateMap({
            source: id,
            file: `${id}.map`,
            includeContent: true
          })
        };
      }
    } catch (error) {
      this.logger.error(error, {
        timestamp: true
      });
    }
  }
  blockReplaceHandler(_, ...args) {
    const {
      isNot: startNot,
      conditional,
      code
    } = this.parseBlockReplaceHandlerParams(args);
    const conditionals = this.parseConditional(conditional);
    const isKeep = conditionals.some((subConditional) => {
      const isNot = startNot !== subConditional.isNot;
      const value = this.env[subConditional.key];
      if (!Object.hasOwn(this.env, subConditional.key)) {
        this.logger.warn(
          `No '${subConditional.key}' propertie in Vite environment variables`,
          {
            timestamp: true
          }
        );
      }
      if (subConditional.value === void 0) {
        return !!value !== isNot;
      } else {
        return String(value) === subConditional.value !== isNot;
      }
    });
    const { ifCode, elseCode } = this.parseElse(code);
    return isKeep ? ifCode : elseCode;
  }
  parseBlockReplaceHandlerParams([$1, $2, $3]) {
    return {
      isNot: !!$1,
      conditional: $2,
      code: $3
    };
  }
  parseConditional(conditional) {
    return conditional.split("||").map((subConditional) => {
      const [_, startNot, key, isNot, value] = subConditional.match(
        /^(!?)(\S+?)(?:(!?)(?:=)(\S+))?$/
      );
      return {
        key,
        isNot: !!startNot !== !!isNot,
        value
      };
    });
  }
  parseElse(code) {
    const [_, ifCode, elseCode] = code?.match(
      /^([\s\S]*?).*#v-else.*[\r\n]{1,2}([\s\S]*)$/
    ) ?? [code, "", ""];
    return {
      ifCode: ifCode ? ifCode : code,
      elseCode: elseCode ? elseCode : ""
    };
  }
  createSource(code, id) {
    return new MagicString(code, {
      filename: id
    });
  }
}

const resolveOptions = (userOptions) => {
  return {
    include: ["**/*"],
    exclude: [],
    ...userOptions
  };
};
const VitePluginConditionalCompile = (userOptions = {}) => {
  const options = resolveOptions(userOptions);
  const ctx = new Context(options);
  return {
    name: "vite-plugin-conditional-compile",
    enforce: "pre",
    configResolved(_config) {
      ctx.setEnv(_config);
    },
    transform: (code, id) => ctx.transform(code, id)
  };
};

module.exports = VitePluginConditionalCompile;
